import 'package:flutter/material.dart';
import 'package:viqcore_flutter/src/app.dart';
import 'package:viqcore_flutter/src/http_client/api_service.dart';

@immutable
class Pagination<T> {
  final List<T> results;
  final PageInfo pageInfo;

  const Pagination({required this.results, required this.pageInfo});

  factory Pagination.fromJson(
          Map<String, dynamic> json, ApiResponseTransformer<T> transformer) =>
      Pagination(
        results: json['results'].map<T>((data) => transformer(data)).toList(),
        pageInfo: PageInfo.fromJson(json['pageInfo']),
      );

  static List fragment(List fields) =>
      ['results'.gql(fields), 'pageInfo'.gql(PageInfo.fragment)];

  @override
  String toString() {
    return 'Pagination(results: $results, pageInfo: $pageInfo )';
  }
}

@immutable
class PageInfo {
  final bool endOfPage;
  final int totalCount;

  factory PageInfo.fromJson(Map<String, dynamic> json) => PageInfo(
        endOfPage: json['endOfPage'] as bool,
        totalCount: json['totalCount'] as int,
      );

  const PageInfo({required this.endOfPage, required this.totalCount});

  static List get fragment => ['endOfPage', 'totalCount'];

  @override
  String toString() {
    return 'PageInfo(endOfPage: $endOfPage, totalCount: $totalCount )';
  }
}

@immutable
class PagingInput {
  final int size;
  final int page;

  const PagingInput({this.size = 20, required this.page});

  @override
  String toString() {
    return 'PagingInput(size: $size, page: $page )';
  }

  Map<String, dynamic> toMap() => {'size': size, 'page': page};
}
