import 'package:flutter/material.dart';

/// Signature of callbacks that have no arguments and return bool.
typedef BoolCallback = bool Function();

@immutable
class UpgraderSettings {
  final String appstoreUrl;
  final String? title;
  final String? updateVersion;
  final bool forceUpdate;
  final String? buttonTitleLater;
  final String? buttonTitleUpdate;
  final int daysToAlertAgain;
  final bool debugDisplayAlways;
  final bool debugDisplayOnce;
  final bool debugLogging;
  final String? prompt;
  final BoolCallback? onIgnore;
  final BoolCallback? onLater;
  final BoolCallback? onUpdate;

  const UpgraderSettings({
    required this.appstoreUrl,
    this.title,
    this.updateVersion,
    this.forceUpdate = false,

    /// The later button title, which defaults to ```Later```
    this.buttonTitleLater,

    /// The update button title, which defaults to ```Update Now```
    this.buttonTitleUpdate,

    /// Days until alerting user again
    this.daysToAlertAgain = 0,

    /// For debugging, always force the upgrade to be available.
    this.debugDisplayAlways = false,

    /// For debugging, display the upgrade at least once once.
    this.debugDisplayOnce = false,

    /// Enable print statements for debugging.
    this.debugLogging = false,

    /// = 'Would you like to update it now?';
    this.prompt,

    /// Called when the ignore button is tapped or otherwise activated.
    /// Return false when the default behavior should not execute.
    this.onIgnore,

    /// Called when the ignore button is tapped or otherwise activated.
    /// Return false when the default behavior should not execute.
    this.onLater,

    /// Called when the ignore button is tapped or otherwise activated.
    /// Return false when the default behavior should not execute.
    this.onUpdate,
  });

  @override
  String toString() {
    return 'UpgraderSettings(appstoreUrl: $appstoreUrl, title: $title, updateVersion: $updateVersion, forceUpdate: $forceUpdate, buttonTitleLater: $buttonTitleLater, buttonTitleUpdate: $buttonTitleUpdate, daysToAlertAgain: $daysToAlertAgain, debugDisplayAlways: $debugDisplayAlways, debugDisplayOnce: $debugDisplayOnce, debugLogging: $debugLogging, prompt: $prompt)';
  }
}
