import 'package:intl/intl.dart';

extension DateFormatter on DateTime {
  String toDate() {
    return DateFormat('dd MMM').format(toLocal());
  }

  String toShortDate() {
    return DateFormat('dd/MM/yyyy').format(toLocal());
  }

  String toShortDateWithDay() {
    return DateFormat('EEEE, dd/MM/yyyy').format(toLocal());
  }

  String toShortDateTime() {
    return DateFormat('dd/MM/yyyy HH:mm').format(toLocal());
  }

  String toTimeFormat() {
    return DateFormat('h a').format(toLocal());
  }

  String toShortTime() {
    return DateFormat('h:mm a').format(toLocal());
  }

  String toMonth() {
    return DateFormat('MMM').format(toLocal());
  }

  String toMonthYear() {
    return DateFormat('MMM yyyy').format(toLocal());
  }

  String toYear() {
    return DateFormat('yyyy').format(toLocal());
  }

  String toDay() {
    return DateFormat('dd').format(toLocal());
  }

  String toTime() {
    return DateFormat('HH:mm').format(toLocal());
  }

  String toLongDate() {
    return DateFormat('MMM dd, yyyy').format(toLocal());
  }

  String toLongDateTime() {
    return DateFormat('MMM dd, yyyy h:mm a').format(toLocal());
  }

  String toShortDay() {
    return DateFormat('EEE').format(toLocal());
  }

  DateTime getDate() {
    return DateTime(year, month, day);
  }

  bool isToday() {
    final now = DateTime.now();
    final today = DateTime(now.year, now.month, now.day);

    return DateTime(year, month, day) == today;
  }

  bool isSameYear() {
    final now = DateTime.now();
    final thisYear = DateTime(now.year);

    return DateTime(year) == thisYear;
  }

  bool isDateEqual(DateTime date) {
    final target = DateTime(date.year, date.month, date.day);

    return DateTime(year, month, day) == target;
  }
}
