import 'dart:io';

import 'package:cookie_jar/cookie_jar.dart';
import 'package:dio/dio.dart';
import 'package:dio/io.dart';
import 'package:dio_cookie_manager/dio_cookie_manager.dart';
import 'package:get_it/get_it.dart';
import 'package:path_provider/path_provider.dart';

extension ApiServiceDioAdapter on Dio {
  void allowSelfSignedCert() {
    (httpClientAdapter as IOHttpClientAdapter).createHttpClient = () {
      final client = HttpClient()
        ..idleTimeout = const Duration(seconds: 3)
        ..badCertificateCallback =
            (X509Certificate cert, String host, int port) => true;
      return client;
    };
  }

  void allowWithCredential() {}

  Future<void> enableCookieManager() async {
    final appDocDir = await getApplicationDocumentsDirectory();
    final cj =
        PersistCookieJar(storage: FileStorage('${appDocDir.path}/.cookies/'));
    final cookieManager = CookieManager(cj);
    interceptors.add(cookieManager);
    GetIt.I.registerSingleton(cookieManager);
  }

  void clearCookies() {
    if (GetIt.I.isRegistered<CookieManager>()) {
      final cookieManager = GetIt.I<CookieManager>();
      cookieManager.cookieJar.deleteAll();
    }
  }
}
