part of 'api.dart';

@immutable
class Optional<T> {
  final T? value;

  const Optional({required this.value});

  void ifPresent(void Function(T value) action, {void Function()? orElse}) {
    if (value != null) {
      action.call(value as T);
    } else {
      orElse?.call();
    }
  }

  factory Optional.fromJson(Map<String, Object?> json) =>
      Optional(value: json['value'] as T?);

  @override
  bool operator ==(dynamic other) {
    return identical(this, other) ||
        (other.runtimeType == runtimeType &&
            other is Optional &&
            (identical(other.value, value) || other.value == value));
  }

  @override
  int get hashCode => Object.hash(runtimeType, value);

  @override
  String toString() {
    return 'Optional(value: $value)';
  }
}
