part of 'api.dart';

class RestQuery {
  // final String action;
  final String url;
  final dynamic body;
  final Map<String, dynamic>? queryParameters;
  ApiService? _client;

  Map<String, String>? _headers;
  Map<String, dynamic>? _extra;
  Duration? _timeout;
  ResponseType? _responseType;

  RestQuery(this.url, {this.body, this.queryParameters});

  RestQuery external({ApiService? client}) {
    _client = client ?? App<API>();
    return this;
  }

  RestQuery timeout(Duration value) {
    _timeout = value;
    return this;
  }

  RestQuery headers(Map<String, String> value) {
    _headers = value;
    return this;
  }

  RestQuery responseType(ResponseType responseType) {
    _responseType = responseType;
    return this;
  }

  RestQuery extra(Map<String, String> value) {
    _extra = value;
    return this;
  }

  Future<ApiResponse<T>> get<T>({
    Map<String, String>? headers,
    Map<String, dynamic>? extra,
    Duration? timeout,
    ProgressCallback? onReceiveProgress,
    ApiResponseTransformer<T>? transformer,
  }) {
    return _request(
      'GET',
      headers: headers,
      extra: extra,
      timeout: timeout,
      onReceiveProgress: onReceiveProgress,
      transformer: transformer,
    );
  }

  Future<ApiResponse<T>> post<T>({
    Map<String, String>? headers,
    Map<String, dynamic>? extra,
    Duration? timeout,
    ProgressCallback? onSendProgress,
    ApiResponseTransformer<T>? transformer,
  }) {
    return _request(
      'POST',
      headers: headers,
      extra: extra,
      timeout: timeout,
      onSendProgress: onSendProgress,
      transformer: transformer,
    );
  }

  Future<ApiResponse<T>> _request<T>(
    String method, {
    Map<String, String>? headers,
    Map<String, dynamic>? extra,
    Duration? timeout,
    Options? options,
    ProgressCallback? onSendProgress,
    ProgressCallback? onReceiveProgress,
    ApiResponseTransformer<T>? transformer,
  }) async {
    _headers ??= headers;
    _extra ??= extra;
    _timeout ??= timeout;
    _client ??= App<API>();

    return _client!.request(method, url,
        data: body is List<dynamic> ? RestBody.params(body) : body,
        queryParameters: queryParameters,
        options: Options(
            headers: _headers,
            responseType: _responseType,
            sendTimeout: _timeout,
            extra: _extra),
        onSendProgress: onSendProgress,
        onReceiveProgress: onReceiveProgress,
        transformer: transformer);
  }
}

class RestBody {
  final Map<String, dynamic> data = {};
  bool _formDataMode;
  bool get formDataMode => _formDataMode;

  RestBody({bool formData = false}) : _formDataMode = formData;

  void _addParam(String name, dynamic value) {
    if (value != null) {
      _formDataMode = _formDataMode || value is XFile || value is List<XFile>;

      data[name] = value;
    }
  }

  void addMap(Map<String, dynamic> map) {
    map.forEach(_addParam);
  }

  /// param can be [FieldBase], [Map<String, dynamic>] or
  /// [MapEntry<String, dynamic>]
  void add(List params) {
    for (final item in params) {
      if (item is Map<String, dynamic>) {
        addMap(item);
      } else if (item is MapEntry<String, dynamic>) {
        _addParam(item.key, item.value);
      }
    }
  }

  static RestBody params(List<dynamic> params) {
    return RestBody()..add(params);
  }
}
