import 'package:dio/dio.dart';

import 'map_helper.dart';

extension DioRequestOptionsHelper on RequestOptions {
  bool get isGQL => extra.hasFlag('GQL');
  bool get isExternal => extra.hasFlag('EXTERNAL');
  bool get isRenewCredential => extra.hasFlag('RENEW_CREDENTIAL');
  bool get isSignOut => extra.hasFlag('SIGN_OUT');
  bool get isRetry => extra.hasFlag('RETRY');
  void setRetryFlag() => extra['RETRY'] = true;
}

extension DioResponseHelper on Response {
  bool get gqlErrors => gqlErrorCode != null && data?['errors'] != null;

  String? get gqlErrorCode => requestOptions.isGQL && data is Map
      ? data['errors']?.first?['extensions']?['code']
      : null;

  String? get gqlErrorMessage => data?['errors']?.first?['message'];

  bool get isUnauthorized {
    return !extra.hasFlag('RENEW_CREDENTIAL') &&
        ((statusCode == 401) ||
            (gqlErrors && gqlErrorCode == 'UNAUTHORIZED_ACCESS'));
  }

  bool get noContent => statusCode == 204;
}
