import 'dart:async';

import 'package:viqcore_flutter/src/http_client/utils/nullable_string_helper.dart';

import '../../app.dart';

// Cookie headers from RFC 6265.
const setCookieHeader = "set-cookie";
const authorizationHeader = "authorization";

class WebCookieManager extends Interceptor {
  late final String _key;
  WebCookieManager() {
    _key = 'webcookiemgr/access-token';
  }

  @override
  void onRequest(RequestOptions options, RequestInterceptorHandler handler) {
    var token = App.box.get(_key);
    if (token.isNotNullOrEmpty) {
      final checksum = Crypto.checkSum(token!);
      options.headers[authorizationHeader] = 'viqcore $checksum $token';
    }
    options.headers['web-no-cookie'] = true;
    handler.next(options);
  }

  @override
  void onResponse(Response response, ResponseInterceptorHandler handler) {
    _saveCookies(response).then((_) => handler.next(response)).catchError(
      (dynamic e, StackTrace s) {
        final err = DioException(
          requestOptions: response.requestOptions,
          error: e,
          stackTrace: s,
        );
        handler.reject(err, true);
      },
    );
  }

  @override
  void onError(DioException err, ErrorInterceptorHandler handler) {
    if (err.response != null) {
      _saveCookies(err.response!).then((_) => handler.next(err)).catchError(
        (dynamic e, StackTrace s) {
          final error = DioException(
            requestOptions: err.response!.requestOptions,
            error: e,
            stackTrace: s,
          );
          handler.next(error);
        },
      );
    } else {
      handler.next(err);
    }
  }

  void clear() {
    App.box.delete(_key);
  }

  Future<void> _saveCookies(Response response) async {
    final accessToken = response.headers.value('access-token');
    if (accessToken == null) {
      return;
    }
    if (accessToken == "") {
      clear();
    } else {
      App.box.put(_key, Uri.decodeFull(accessToken));
    }
  }
}
