import 'dart:convert';
import 'dart:typed_data';

import 'package:crypto/crypto.dart';
import 'package:flutter/cupertino.dart';
import 'package:pointycastle/export.dart';

import '../app.dart';

@immutable
class Crypto {
  static String checkSum(String data, {String? signingKey}) {
    final key = utf8.encode(signingKey ?? App<API>().apiKey ?? '');
    final bytes = utf8.encode(data);

    final hmacSha256 = Hmac(sha256, key);
    final digest = hmacSha256.convert(bytes);
    return digest.toString().replaceAll('-', '').toLowerCase();
  }

  static String aesDecrypt(String encryptedData, String ivKey) {
    // Ensure key length
    final hashedKeyBytes =
        sha256.convert(utf8.encode(App<API>().apiKey!)).bytes;

    // Ensure IV length (use first 16 bytes of the hash)
    final hashedIVBytes = utf8.encode(ivKey).sublist(0, 16);

    final value = String.fromCharCodes((_aesCbcDecrypt(
        Uint8List.fromList(hashedKeyBytes),
        Uint8List.fromList(hashedIVBytes),
        base64.decode(encryptedData))));

    return value;
  }

  static Uint8List _aesCbcDecrypt(
      Uint8List key, Uint8List iv, Uint8List cipherText) {
    assert([128, 192, 256].contains(key.length * 8));
    assert(128 == iv.length * 8);
    assert(128 == cipherText.length * 8);

    // Create a CBC block cipher with AES, and initialize with key and IV

    final cbc = CBCBlockCipher(AESEngine())
      ..init(false, ParametersWithIV(KeyParameter(key), iv)); // false=decrypt

    // Decrypt the cipherText block-by-block

    final paddedPlainText = Uint8List(cipherText.length); // allocate space

    var offset = 0;
    while (offset < cipherText.length) {
      offset += cbc.processBlock(cipherText, offset, paddedPlainText, offset);
    }
    assert(offset == cipherText.length);

    return paddedPlainText.sublist(
        0, paddedPlainText.length - PKCS7Padding().padCount(paddedPlainText));
  }
}
